//
//  DraftView.m
//  Fast Draft
//
//  Created by AR on 21.02.13.
//  Copyright (c) 2013 Geometros Company. All rights reserved.
//

#import "DraftView.h"
#include "../sgCore/sgCore.h"
#import "QuartzCore/QuartzCore.h"

#import  "SnapSystemLayer.h"

static bool load_fonts()
{
	static bool wasLoad = false;
	if (wasLoad)
		return true;
    
    NSArray * fontsNames = [NSArray arrayWithObjects:@"COMPLEX",
                                                     @"GOTHICE",
                                                     @"GOTHICG",
                                                     @"GOTHICI",
                                                     @"ITALIC",
                                                     @"MONOTXT",
                                                     @"ROMANC",
                                                     @"ROMAND",
                                                     @"ROMANT",
                                                     @"SCRIPTC",
                                                     @"SCRIPTS",
                                                     @"SYMATH",
                                                        nil];
    
    for (NSString * curFontName in fontsNames)
    {
        NSString* path = [[NSBundle mainBundle] pathForResource:curFontName ofType:@"SHX"];
        const char* cString = [path cStringUsingEncoding:NSASCIIStringEncoding];
    
    
	    sgCFont* fnt1 = sgCFont::LoadFont(cString,NULL,0);
	    if (!fnt1)
	    {
		    return false;
	    }
	    if (!sgFontManager::AttachFont(fnt1))
	    {
		    sgCFont::UnloadFont(fnt1);
		    return false;
	    }
    }
    
    
    sgFontManager::SetCurrentFont(0);
    
	wasLoad = true;
	return true;
}

#define    SNAP_LAYER_INDEX     0
#define    SCELE_LAYER_INDEX     (SNAP_LAYER_INDEX+1)
#define    CURRENT_LAYER_INDEX   (SNAP_LAYER_INDEX+2)



@implementation DraftView
{
   // UIImage*   mainBckgrImage;
    UIImage*   mainBckgrLightImage;
        
    SceneDrawer*  sceneLayer;
    CurrentObjectDrawer*  currentObjectLayer;
    SnapSystemLayer*      snapSystemLayer;

    id<ICommandsManager>   commandsManager;

}

- (id)initWithCoder:(NSCoder*)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self) {
       // mainBckgrImage = [UIImage imageNamed:@"blueprintGrid.png"];
        mainBckgrLightImage = [UIImage imageNamed:@"blueprintLight.png"];
        
        commandsManager = nil;
        
        self.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"blueprintGrid.png"]];
        self.contentMode = UIViewContentModeRedraw;
        
        sgInitKernel();
        
        load_fonts();
        
        [Drawer InitStyle];

        CGRect screenRect = [[UIScreen mainScreen] bounds];
        snapSystemLayer = [[SnapSystemLayer alloc] initWithSize:fmax(screenRect.size.width, screenRect.size.height)];
        
        CALayer *snapLayer = [CALayer layer];
        snapLayer.frame = CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height);
        snapLayer.delegate = snapSystemLayer;
        [self.layer addSublayer:snapLayer];
        [snapLayer setNeedsDisplay];

        
        sceneLayer = [[SceneDrawer alloc] init];
        
        CALayer *drawLayer = [CALayer layer];
        drawLayer.frame = CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height);
        drawLayer.delegate = sceneLayer;
        [self.layer addSublayer:drawLayer];
        [drawLayer setNeedsDisplay];
        
    
        currentObjectLayer = [[CurrentObjectDrawer alloc] init];
        
        CALayer *curObjLayer = [CALayer layer];
        curObjLayer.frame = CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height);
        curObjLayer.delegate = currentObjectLayer;
        [self.layer addSublayer:curObjLayer];
        [curObjLayer setNeedsDisplay];

    }
    return self;
}

-(void) setCommandsManager:(id<ICommandsManager>)manag
{
    commandsManager = manag;
    [commandsManager  setSnapManager:snapSystemLayer];

    [currentObjectLayer setCommandsManager:manag];
}

-(void) redrawAll
{
    [[self.layer.sublayers objectAtIndex:SNAP_LAYER_INDEX] setNeedsDisplay];
    [[self.layer.sublayers objectAtIndex:SCELE_LAYER_INDEX] setNeedsDisplay];
    [[self.layer.sublayers objectAtIndex:CURRENT_LAYER_INDEX] setNeedsDisplay];
}

-(void) Clear
{
    sgGetScene()->Clear();
    [snapSystemLayer Clear];
    
     [self redrawAll];
}

- (IBAction)SaveImage:(id)sender
{
    UIGraphicsBeginImageContextWithOptions(self.bounds.size, YES, 0);
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    UIImage* backIm = [UIImage imageNamed:@"blueprintGrid.png"];
    CGImageRef image = CGImageRetain(backIm.CGImage);
    
    CGContextClipToRect(context, CGRectMake(0.0, 0.0, self.bounds.size.width, self.bounds.size.height));
    
    CGContextDrawTiledImage(context, CGRectMake(0.0, 0.0, backIm.size.width, backIm.size.height), image);
    
    CGContextDrawImage(context, CGRectMake(0.0, 0.0, self.bounds.size.width, self.bounds.size.height), mainBckgrLightImage.CGImage);
    
    [sceneLayer drawLayer:nil inContext:context];
    
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    UIImageWriteToSavedPhotosAlbum(newImage, self,nil, nil);
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    [[self.layer.sublayers objectAtIndex:SNAP_LAYER_INDEX] setFrame:self.bounds];
    [[self.layer.sublayers objectAtIndex:SCELE_LAYER_INDEX] setFrame:self.bounds];
    [[self.layer.sublayers objectAtIndex:CURRENT_LAYER_INDEX] setFrame:self.bounds];

    [self redrawAll];


}


// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect
{
    [mainBckgrLightImage drawInRect:rect];
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch *touch = [touches anyObject];
    CGPoint pnt = [touch locationInView:self];
    
    [[commandsManager getCommand:[commandsManager getCurrentCommand]] touchBegan:pnt];
    
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    
    UITouch *touch = [touches anyObject];
    CGPoint pnt = [touch locationInView:self];
    
    [[commandsManager getCommand:[commandsManager getCurrentCommand]] touchMoved:pnt];

    [[self.layer.sublayers objectAtIndex:CURRENT_LAYER_INDEX] setNeedsDisplay];
    [[self.layer.sublayers objectAtIndex:SNAP_LAYER_INDEX] setNeedsDisplay];

}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event;
{
    UITouch *touch = [touches anyObject];
    CGPoint pnt = [touch locationInView:self];
    
    [[commandsManager getCommand:[commandsManager getCurrentCommand]] touchEnded:pnt];

    [self redrawAll];


}

@end




/*****************************/
@implementation CurrentObjectDrawer
{
    UIImage*   handImage;
    
    id<ICommandsManager>   commandsManager;

}

- (id)init
{
    self = [super init];
    if (self) {
        handImage = [UIImage imageNamed:@"hand.png"];
    }
    return self;
}

-(void) setCommandsManager:(id<ICommandsManager>)manag;
{
    commandsManager = manag;
}

-(void) drawLayer: (CALayer*) layer inContext: (CGContextRef) currentContext
{
    CGContextSetRGBStrokeColor(currentContext, 1.0f, 1.0f, 0.0f, 1.0f);
    [Drawer ApplyStyleToContext:currentContext];
    
    [[commandsManager getCommand:[commandsManager getCurrentCommand]] draw:currentContext];
}



@end







/*****************************/
@implementation SceneDrawer
- (id)init
{
    self = [super init];
    if (self) {
              
    }
    return self;
}

-(void) drawLayer: (CALayer*) layer inContext: (CGContextRef) context
{
    CGContextSetLineWidth(context, 4);
    CGContextSetRGBStrokeColor(context, 1.0f, 1.0f, 1.0f, 1.0f);
    CGContextSetLineCap(context, kCGLineCapRound);
    float normal[1]={1};
    CGContextSetLineDash(context,0,normal,0);

    sgCObject*  curObj = sgGetScene()->GetObjectsList()->GetHead();
	while (curObj)
	{
        [Drawer drawObject:curObj inContext:context];
		curObj = sgGetScene()->GetObjectsList()->GetNext(curObj);
	}
	
    
}

@end

