//
//  SnapSystemLayer.m
//  Fast Draft
//
//  Created by AR on 01.03.13.
//  Copyright (c) 2013 Geometros Company. All rights reserved.
//

#import "SnapSystemLayer.h"
#include "../sgCore/sgCore.h"

#define SNAP_RECT_X_COUNT      10
#define SNAP_RECT_Y_COUNT      10


#define SNAP_AREA_SIZE         10


@interface OneSnapRect : NSObject

-(id)      initWithRect:(CGRect)rct;

-(void)       Clear;

-(CGRect)  getRect;

-(void)   addPoint:(SG_POINT)pnt;
-(void)   addLineFromPoint:(SG_POINT)pnt withDirection:(SG_VECTOR)lineDir;

-(BOOL)   isPointInside:(SG_POINT)pnt;
-(BOOL)   isLineIntersectFromPoint:(SG_POINT)pnt WithDirection:(SG_VECTOR)dir;

-(BOOL)   getHotPoint:(CGPoint&)hotPoint;
-(BOOL)   getHotLineFrom:(CGPoint&)p1 To:(CGPoint&)p2;
 
-(SG_POINT)  Snap:(SG_POINT)pnt;

-(void)    draw:(CGContextRef) context;
 
@end



@implementation SnapSystemLayer
{
    NSMutableArray*    snapsRects;
    CGSize             rectsSize;
    int                hotRectIndex;
}

- (id)initWithSize:(int)maxSize
{
    self = [super init];
    if (self)
    {
        int  oneRectW = maxSize/SNAP_RECT_X_COUNT;
        int  oneRectH = maxSize/SNAP_RECT_Y_COUNT;
        
        rectsSize.width = oneRectW;
        rectsSize.height = oneRectH;
        
        snapsRects = [[NSMutableArray alloc] init];
        
        for (int i=0;i<SNAP_RECT_X_COUNT;i++)
        {
            for (int j=0; j<SNAP_RECT_Y_COUNT; j++)
            {
                [snapsRects addObject:[[OneSnapRect alloc] initWithRect:CGRectMake(i*oneRectW, j*oneRectH, oneRectW, oneRectH)]];
            }
        }
        
        hotRectIndex = -1;

    }
    
    return self;
}

-(void)       Clear
{
    for (int i=0;i<snapsRects.count;i++)
    {
        OneSnapRect*  currRct = [snapsRects objectAtIndex:i];
        
        [currRct Clear];
    }
}

-(void) drawLayer: (CALayer*) layer inContext: (CGContextRef) context
{
    if (hotRectIndex>=0 && hotRectIndex<snapsRects.count)
    {
        CGContextSetLineWidth(context, 1);
        CGContextSetRGBStrokeColor(context, 1.0f, 1.0f, 0.0f, 0.8f);
        float dash[2]={3 ,3};
        CGContextSetLineDash(context,0,dash,2);
        
        OneSnapRect*  currRct = [snapsRects objectAtIndex:hotRectIndex];
        
        CGPoint p1, p2;
        
        if ([currRct getHotPoint:p1])
        {
            CGContextBeginPath(context);
            CGContextStrokeEllipseInRect(context, CGRectMake(p1.x - SNAP_AREA_SIZE, p1.y - SNAP_AREA_SIZE, 2*SNAP_AREA_SIZE, 2*SNAP_AREA_SIZE));
            CGContextStrokePath(context);
        }
        else  if ([currRct getHotLineFrom:p1 To:p2])
        {
            CGContextBeginPath(context);
            
            CGContextMoveToPoint(context, p1.x, p1.y);
            CGContextAddLineToPoint(context, p2.x, p2.y);
            
            CGContextStrokePath(context);
        }
    }
    
    /*CGContextSetLineWidth(context, 1);
    CGContextSetRGBStrokeColor(context, 1.0f, 1.0f, 1.0f, 0.5f);
    float dash[2]={3 ,3}; 
    CGContextSetLineDash(context,0,dash,2);
    
    for (int i=0;i<snapsRects.count;i++)
    {
        OneSnapRect*  currRct = [snapsRects objectAtIndex:i];

        [currRct draw:context];
    }*/
    
    
}

-(int)  getRectIndexFromPoint:(CGPoint)pnt
{
    int xInd = (int) ( pnt.x / rectsSize.width );
    int yInd = (int) ( pnt.y / rectsSize.height );

    
    return  (xInd*SNAP_RECT_Y_COUNT + yInd);
}

-(void) addPoint:(CGPoint)pnt
{
    int pntRctInd = [self getRectIndexFromPoint:pnt];
    if (pntRctInd<0 || pntRctInd>=snapsRects.count)
        return;
    
    OneSnapRect*  currRct = [snapsRects objectAtIndex:pntRctInd];
    
    SG_POINT   sgPnt = {pnt.x, pnt.y, 0};
    SG_VECTOR  xAxe = {1, 0, 0};
    SG_VECTOR  yAxe = {0, 1, 0};
    [currRct addPoint:sgPnt];
    
    int lineBegInd = [self getRectIndexFromPoint:CGPointMake(10, pnt.y)];
    for(int i=lineBegInd; i<snapsRects.count; i+=SNAP_RECT_Y_COUNT)
    {
            OneSnapRect*  currRct2 = [snapsRects objectAtIndex:i];
            [currRct2 addLineFromPoint:sgPnt withDirection:xAxe];
    }
    
    lineBegInd = [self getRectIndexFromPoint:CGPointMake(pnt.x, 10)];
    for(int i=0; i<SNAP_RECT_Y_COUNT; i++)
    {
        if ((lineBegInd+i)<snapsRects.count)
        {
         OneSnapRect*  currRct2 = [snapsRects objectAtIndex:(lineBegInd+i)];
         [currRct2 addLineFromPoint:sgPnt withDirection:yAxe];
        }

    }
}

-(void) addLineFrom:(CGPoint)pnt1 To:(CGPoint)pnt2
{
    SG_POINT   sgP = {pnt1.x, pnt1.y, 0};
    SG_VECTOR  sgDir = {pnt2.x - pnt1.x, pnt2.y - pnt1.y, 0};
    if (!sgSpaceMath::NormalVector(sgDir))
        return;
    for (int i=0;i<snapsRects.count;i++)
    {
        OneSnapRect*  currRct = [snapsRects objectAtIndex:i];
        [currRct addLineFromPoint:sgP withDirection:sgDir];
    }
}

-(CGPoint)    Snap:(CGPoint)pnt
{
    if (pnt.x==-1 && pnt.y==-1)
    {
        hotRectIndex = -1;
        return pnt;
    }
    hotRectIndex = [self getRectIndexFromPoint:pnt];
    
    if (hotRectIndex>-1 && hotRectIndex<snapsRects.count)
    {
        OneSnapRect*  currRct = [snapsRects objectAtIndex:hotRectIndex];
        SG_POINT  wasPnt = {pnt.x, pnt.y, 0};
        wasPnt = [currRct Snap:wasPnt];
        return CGPointMake(wasPnt.x, wasPnt.y);
    }
    
    return  pnt;
}


@end











/**************************/
typedef  struct
{
    SG_POINT    point;
    SG_VECTOR   direction;
}   POINT_WITH_DIRECTION;

bool  line_rect_intersect_cohen_sutherland(double lineX1, double lineY1, double lineX2, double lineY2, double rectX, double rectY, double rectWidth, double rectHeight);

@implementation OneSnapRect
{
    CGRect           mainRect;
    NSMutableArray*  allPoints;
    NSMutableArray*  allLines;
    
    int              hotPointIndex;
    int              hotLineIndex;
}

- (id)initWithRect:(CGRect)rct
{
    self = [super init];
    if (self) {
        mainRect = rct;
        allPoints = [[NSMutableArray alloc] init];
        allLines = [[NSMutableArray alloc] init];
        
        hotLineIndex = hotPointIndex = -1;
    }
    
    return self;
}

-(void)       Clear
{
    [allPoints removeAllObjects];
    [allLines removeAllObjects];
    
    hotLineIndex = hotPointIndex = -1;
}

-(CGRect)  getRect
{
    return mainRect;
}

-(void)   addPoint:(SG_POINT)pnt
{
    NSValue *value = [NSValue value:&pnt withObjCType:@encode(SG_POINT)];
    [allPoints addObject:value];
    //NSLog(@"Added point - %f,  %f,  %f", pnt.x, pnt.y, pnt.z);
}

-(void)   addLineFromPoint:(SG_POINT)pnt withDirection:(SG_VECTOR)lineDir
{
    if ([self isLineIntersectFromPoint:pnt WithDirection:lineDir])
    {
       POINT_WITH_DIRECTION tmpPD;
       tmpPD.point = pnt;
       tmpPD.direction = lineDir;
    
       NSValue *value = [NSValue value:&tmpPD withObjCType:@encode(POINT_WITH_DIRECTION)];
       [allLines addObject:value];
        
      //  NSLog(@"Line - %f,  %f,  %f; Dir - %f,  %f,  %f; Rect: %f, %f", pnt.x, pnt.y, pnt.z, lineDir.x, lineDir.y, lineDir.z, mainRect.origin.x, mainRect.origin.y);
    }
}

-(BOOL)   isPointInside:(SG_POINT)pnt
{
    return CGRectContainsPoint(mainRect, CGPointMake(pnt.x, pnt.y));
}

-(BOOL)   isLineIntersectFromPoint:(SG_POINT)pnt WithDirection:(SG_VECTOR)dir
{
    if ([self isPointInside:pnt])
        return  YES;
    
    SG_POINT   p2_1 = {pnt.x + dir.x*10000.0f, pnt.y + dir.y*10000.0f, 0};
    SG_POINT   p2_2 = {pnt.x - dir.x*10000.0f, pnt.y - dir.y*10000.0f, 0};

    return line_rect_intersect_cohen_sutherland(p2_1.x, p2_1.y,
                                                p2_2.x, p2_2.y,
                                                mainRect.origin.x, mainRect.origin.y,
                                                mainRect.size.width, mainRect.size.height);
    
}


-(SG_POINT)  Snap:(SG_POINT)pnt
{
    hotLineIndex = hotPointIndex = -1;

    for (int i=0; i<[self getPointsCount];i++)
    {
        SG_POINT curP = [self getPointAtIndex:i];
        if (sgSpaceMath::PointsDistance(pnt, curP)<SNAP_AREA_SIZE)
        {
            hotPointIndex = i;
            return curP;
        }
    }
    
    for (int i=0; i< [self getLinesCount];i++)
    {
        POINT_WITH_DIRECTION  curPD = [self getLineAtIndex:i];
        SG_POINT  projPnt;
        if (sgSpaceMath::ProjectPointToLineAndGetDist(curPD.point, curPD.direction, pnt, projPnt)<SNAP_AREA_SIZE)
        {
            hotLineIndex = i;
            return  projPnt;
        }
    }
    return pnt;
}

-(BOOL)   getHotPoint:(CGPoint&)hotPoint
{
    if (hotPointIndex<0 || hotPointIndex>=allPoints.count)
        return NO;
    
    SG_POINT  sgHot = [self getPointAtIndex:hotPointIndex];
    hotPoint = CGPointMake(sgHot.x, sgHot.y);
    return YES;
}
-(BOOL)   getHotLineFrom:(CGPoint&)p1 To:(CGPoint&)p2
{
    if (hotLineIndex<0 || hotLineIndex>=allLines.count)
        return NO;
    
    POINT_WITH_DIRECTION  sgHot = [self getLineAtIndex:hotLineIndex];
    
    p1 = CGPointMake(sgHot.point.x + sgHot.direction.x*1000.0f, sgHot.point.y + sgHot.direction.y*1000.0f);
    p2 = CGPointMake(sgHot.point.x - sgHot.direction.x*1000.0f, sgHot.point.y - sgHot.direction.y*1000.0f);
    return YES;
}

-(void)    draw:(CGContextRef) context
{
    
    CGPoint p1, p2;

    for (int i=0;i<allPoints.count;i++)
    {
        hotPointIndex = i;
       if ([self getHotPoint:p1])
       {
        CGContextBeginPath(context);
        CGContextStrokeEllipseInRect(context, CGRectMake(p1.x - SNAP_AREA_SIZE/2, p1.y - SNAP_AREA_SIZE/2, SNAP_AREA_SIZE, SNAP_AREA_SIZE));
        CGContextStrokePath(context);
      }
    }
    
    CGContextSaveGState(context);
    CGContextClipToRect(context, mainRect);
    for (int i=0;i<allLines.count;i++)
    {
        hotLineIndex = i;
       if ([self getHotLineFrom:p1 To:p2])
       {
        CGContextBeginPath(context);
        
        CGContextMoveToPoint(context, p1.x, p1.y);
        CGContextAddLineToPoint(context, p2.x, p2.y);
        
        CGContextStrokePath(context);
       }
    }
    
    CGContextRestoreGState(context);
}


-(int)    getPointsCount
{
    return  allPoints.count;
}

-(int)    getLinesCount
{
    return allLines.count;
}

-(SG_POINT)   getPointAtIndex:(int)indx
{
    SG_POINT resP = {0,0,0};
    if (indx>=0 && indx<allPoints.count)
    {
        NSValue *value = [allPoints objectAtIndex:indx];
        [value getValue:&resP];

    }
    return  resP;
}

-(POINT_WITH_DIRECTION)   getLineAtIndex:(int)indx
{
    POINT_WITH_DIRECTION resPD;
    if (indx>=0 && indx<allLines.count)
    {
        NSValue *value = [allLines objectAtIndex:indx];
        [value getValue:&resPD];
        
    }
    return  resPD;
}



@end



#define OUT_LEFT    1
#define OUT_TOP     2
#define OUT_RIGHT   4
#define OUT_BOTTOM  8

int outcode(double pX, double pY, double rectX, double rectY, double rectWidth, double rectHeight)
{
    int out = 0;
    if (rectWidth <= 0) {
        out |= OUT_LEFT | OUT_RIGHT;
    } else if (pX < rectX) {
        out |= OUT_LEFT;
    } else if (pX > rectX + rectWidth) {
        out |= OUT_RIGHT;
    }
    if (rectHeight <= 0) {
        out |= OUT_TOP | OUT_BOTTOM;
    } else if (pY < rectY) {
        out |= OUT_TOP;
    } else if (pY > rectY + rectHeight) {
        out |= OUT_BOTTOM;
    }
    return out;
}

bool  line_rect_intersect_cohen_sutherland(double lineX1, double lineY1, double lineX2, double lineY2, double rectX, double rectY, double rectWidth, double rectHeight)
{
    int out1, out2;
    if ((out2 = outcode(lineX2, lineY2, rectX, rectY, rectWidth, rectHeight)) == 0) {
        return true;
    }
    while ((out1 = outcode(lineX1, lineY1, rectX, rectY, rectWidth, rectHeight)) != 0) {
        if ((out1 & out2) != 0) {
            return false;
        }
        if ((out1 & (OUT_LEFT | OUT_RIGHT)) != 0) {
            double x = rectX;
            if ((out1 & OUT_RIGHT) != 0) {
                x += rectWidth;
            }
            lineY1 = lineY1 + (x - lineX1) * (lineY2 - lineY1) / (lineX2 - lineX1);
            lineX1 = x;
        } else {
            double y = rectY;
            if ((out1 & OUT_BOTTOM) != 0) {
                y += rectHeight;
            }
            lineX1 = lineX1 + (y - lineY1) * (lineX2 - lineX1) / (lineY2 - lineY1);
            lineY1 = y;
        }
    }
    return true;
}